Option Explicit On 
Option Strict On

Imports System
Imports System.Collections
Imports System.ComponentModel
Imports System.Globalization
Imports System.Runtime.InteropServices
Imports System.Runtime.InteropServices.Marshal

Namespace Edanmo.Net

    Public NotInheritable Class Dns

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' DNS Api
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        <System.Security.SuppressUnmanagedCodeSecurity()> _
        Private NotInheritable Class UnsafeNativeMethods

            Private Sub New()
            End Sub

            <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Auto)> _
            Public Structure DNS_A_DATA
                Public IpAddress As Int32
            End Structure

            <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Auto)> _
            Public Structure DNS_MX_DATA
                Public pNameExchange As IntPtr
                Public wPreference As Short
                Public Pad As Short
            End Structure

            <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Auto)> _
            Public Structure DNS_PTR_DATA
                Public pNameHost As IntPtr
            End Structure

            <StructLayout(LayoutKind.Explicit, CharSet:=CharSet.Auto)> _
            Public Structure DnsRecord

                <FieldOffset(0)> Public pNext As IntPtr
                <FieldOffset(4)> Public pName As String
                <FieldOffset(8)> Public wType As Short
                <FieldOffset(10)> Public wDataLength As Short
                <FieldOffset(12)> Public flags As Integer
                <FieldOffset(16)> Public dwTtl As Integer
                <FieldOffset(20)> Public dwReserved As Integer
                <FieldOffset(24)> Public A As DNS_A_DATA
                <FieldOffset(24)> Public NS As DNS_PTR_DATA
                <FieldOffset(24)> Public PTR As DNS_PTR_DATA
                <FieldOffset(24)> Public MX As DNS_MX_DATA

            End Structure

            Public Enum DnsQueryOptions As Integer
                DNS_QUERY_STANDARD = &H0
                DNS_QUERY_ACCEPT_TRUNCATED_RESPONSE = &H1
                DNS_QUERY_USE_TCP_ONLY = &H2
                DNS_QUERY_NO_RECURSION = &H4
                DNS_QUERY_BYPASS_CACHE = &H8
                DNS_QUERY_NO_WIRE_QUERY = &H10
                DNS_QUERY_NO_LOCAL_NAME = &H20
                DNS_QUERY_NO_HOSTS_FILE = &H40
                DNS_QUERY_NO_NETBT = &H80
                DNS_QUERY_WIRE_ONLY = &H100
                DNS_QUERY_RETURN_MESSAGE = &H200
                DNS_QUERY_TREAT_AS_FQDN = &H1000
                DNS_QUERY_DONT_RESET_TTL_VALUES = &H100000
                DNS_QUERY_RESERVED = &HFF000000
            End Enum

            Public Enum DnsQueryTypes As Integer
                DNS_TYPE_A = &H1
                DNS_TYPE_NS = &H2
                DNS_TYPE_MD = &H3
                DNS_TYPE_MF = &H4
                DNS_TYPE_CNAME = &H5
                DNS_TYPE_SOA = &H6
                DNS_TYPE_MB = &H7
                DNS_TYPE_MG = &H8
                DNS_TYPE_MR = &H9
                DNS_TYPE_NULL = &HA
                DNS_TYPE_WKS = &HB
                DNS_TYPE_PTR = &HC
                DNS_TYPE_HINFO = &HD
                DNS_TYPE_MINFO = &HE
                DNS_TYPE_MX = &HF
                DNS_TYPE_TEXT = &H10
                DNS_TYPE_RP = &H11
                DNS_TYPE_AFSDB = &H12
                DNS_TYPE_X25 = &H13
                DNS_TYPE_ISDN = &H14
                DNS_TYPE_RT = &H15
                DNS_TYPE_NSAP = &H16
                DNS_TYPE_NSAPPTR = &H17
                DNS_TYPE_SIG = &H18
                DNS_TYPE_KEY = &H19
                DNS_TYPE_PX = &H1A
                DNS_TYPE_GPOS = &H1B
                DNS_TYPE_AAAA = &H1C
                DNS_TYPE_LOC = &H1D
                DNS_TYPE_NXT = &H1E
                DNS_TYPE_EID = &H1F
                DNS_TYPE_NIMLOC = &H20
                DNS_TYPE_SRV = &H21
                DNS_TYPE_ATMA = &H22
                DNS_TYPE_NAPTR = &H23
                DNS_TYPE_KX = &H24
                DNS_TYPE_CERT = &H25
                DNS_TYPE_A6 = &H26
                DNS_TYPE_DNAME = &H27
                DNS_TYPE_SINK = &H28
                DNS_TYPE_OPT = &H29
                DNS_TYPE_UINFO = &H64
                DNS_TYPE_UID = &H65
                DNS_TYPE_GID = &H66
                DNS_TYPE_UNSPEC = &H67
                DNS_TYPE_ADDRS = &HF8
                DNS_TYPE_TKEY = &HF9
                DNS_TYPE_TSIG = &HFA
                DNS_TYPE_IXFR = &HFB
                DNS_TYPE_AXFR = &HFC
                DNS_TYPE_MAILB = &HFD
                DNS_TYPE_MAILA = &HFE
                DNS_TYPE_ALL = &HFF
                DNS_TYPE_ANY = &HFF
            End Enum

            Public Declare Unicode Function DnsQuery Lib "dnsapi" Alias "DnsQuery_W" ( _
                ByVal pszName As String, _
                ByVal wType As DnsQueryTypes, _
                ByVal options As DnsQueryOptions, _
                ByVal aipServers As Integer, _
                ByRef ppQueryResults As IntPtr, _
                ByVal pReserved As Integer) As Integer

            Public Declare Auto Sub DnsRecordListFree Lib "dnsapi" ( _
                ByVal pRecordList As IntPtr, _
                ByVal FreeType As Integer)

        End Class

        Shared Sub New()

            ' Check if the machine is running Win2K, XP, 2003 Server
            If Environment.OSVersion.Platform <> PlatformID.Win32NT Then
                Throw New NotSupportedException
            ElseIf Environment.OSVersion.Version.Major < 5 Then
                Throw New NotSupportedException
            End If

        End Sub

        Private Sub New()
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the authoritative name servers of a domain.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetAuthoritativeNameServers(ByVal domain As String) As String()

            Dim ptr As IntPtr
            Dim result As Integer
            Dim servers As New ArrayList

            ' Query the DNS server for the NS records
            result = UnsafeNativeMethods.DnsQuery( _
                domain, _
                UnsafeNativeMethods.DnsQueryTypes.DNS_TYPE_NS, _
                UnsafeNativeMethods.DnsQueryOptions.DNS_QUERY_BYPASS_CACHE, _
                0, ptr, 0)

            If result = 0 Then

                Dim ptrNext As IntPtr = ptr

                ' Enumerate all returned records
                Do Until ptrNext.Equals(IntPtr.Zero)

                    ' Get the record from the pointer
                    Dim record As UnsafeNativeMethods.DnsRecord = DirectCast(PtrToStructure(ptrNext, GetType(UnsafeNativeMethods.DnsRecord)), UnsafeNativeMethods.DnsRecord)

                    ' Get only the NS records
                    If record.wType = UnsafeNativeMethods.DnsQueryTypes.DNS_TYPE_NS Then

                        ' Add the host name to the list
                        servers.Add(Marshal.PtrToStringAuto(record.NS.pNameHost))

                    End If

                    ' Get the pointer to the next record
                    ptrNext = record.pNext

                Loop

                ' Release the record list
                UnsafeNativeMethods.DnsRecordListFree(ptr, 0)

            Else

                ' Throw the exception
                Throw New Win32Exception(result)

            End If

            ' Convert the list to an array and return it
            Return DirectCast(servers.ToArray(GetType(String)), String())

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the canonical name of a host.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetCanonicalName(ByVal host As String) As String

            Dim ptr As IntPtr
            Dim result As Integer
            Dim hostName As String

            ' Query the DNS hostName for the CNAME record
            result = UnsafeNativeMethods.DnsQuery( _
                host, _
                UnsafeNativeMethods.DnsQueryTypes.DNS_TYPE_CNAME, _
                UnsafeNativeMethods.DnsQueryOptions.DNS_QUERY_BYPASS_CACHE, _
                0, ptr, 0)

            If result = 0 Then

                Dim ptrNext As IntPtr = ptr

                ' Enumerate all returned records
                Do Until ptrNext.Equals(IntPtr.Zero)

                    ' Get the record from the pointer
                    Dim record As UnsafeNativeMethods.DnsRecord = DirectCast(PtrToStructure(ptrNext, GetType(UnsafeNativeMethods.DnsRecord)), UnsafeNativeMethods.DnsRecord)

                    ' Check if the record type is CNAME
                    If record.wType = UnsafeNativeMethods.DnsQueryTypes.DNS_TYPE_CNAME Then

                        ' Store the host name
                        hostName = Marshal.PtrToStringAuto(record.PTR.pNameHost)

                        Exit Do

                    End If

                    ' Get the pointer to the next record
                    ptrNext = record.pNext

                Loop

                ' Release the record list
                UnsafeNativeMethods.DnsRecordListFree(ptr, 0)

            Else

                ' Throw the exception
                Throw New Win32Exception(result)

            End If

            ' Return the host name 
            Return hostName

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the host name from a IP (reverse lookup)
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetHostName(ByVal ip As System.Net.IPAddress) As String

            Dim ptr As IntPtr
            Dim result As Integer
            Dim hostName As String
            Dim addrByte() As Byte = ip.GetAddressBytes()
            Dim address As String

            ' Convert the IP to IN-ADDR.ARPA format
            address = String.Format( _
                CultureInfo.InvariantCulture, _
                "{0}.{1}.{2}.{3}.IN-ADDR.ARPA", _
                addrByte(3), addrByte(2), addrByte(1), addrByte(0))

            ' Query the PTR records for the host
            result = UnsafeNativeMethods.DnsQuery( _
                address, _
                UnsafeNativeMethods.DnsQueryTypes.DNS_TYPE_PTR, _
                UnsafeNativeMethods.DnsQueryOptions.DNS_QUERY_BYPASS_CACHE Or UnsafeNativeMethods.DnsQueryOptions.DNS_QUERY_NO_LOCAL_NAME, _
                0, ptr, 0)

            If result = 0 Then

                Dim ptrNext As IntPtr = ptr

                ' Enumerate all returned records
                Do Until ptrNext.Equals(IntPtr.Zero)

                    ' Get the record from the pointer
                    Dim record As UnsafeNativeMethods.DnsRecord = DirectCast(PtrToStructure(ptrNext, GetType(UnsafeNativeMethods.DnsRecord)), UnsafeNativeMethods.DnsRecord)

                    If record.wType = UnsafeNativeMethods.DnsQueryTypes.DNS_TYPE_PTR Then

                        ' Return the host name
                        hostName = Marshal.PtrToStringAuto(record.PTR.pNameHost)

                        Exit Do

                    End If

                    ' Get the pointer to the next record
                    ptrNext = record.pNext

                Loop

                ' Release the record list
                UnsafeNativeMethods.DnsRecordListFree(ptr, 0)

            Else

                ' Throw the exception
                Throw New Win32Exception(result)

            End If

            ' Return the host name
            Return hostName

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the  IP addresses of a host
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetIPAddresses(ByVal host As String) As System.Net.IPAddress()

            Dim ptr As IntPtr
            Dim result As Integer
            Dim servers As New ArrayList

            ' Query the DNS server for the A records
            result = UnsafeNativeMethods.DnsQuery( _
                host, _
                UnsafeNativeMethods.DnsQueryTypes.DNS_TYPE_A, _
                UnsafeNativeMethods.DnsQueryOptions.DNS_QUERY_BYPASS_CACHE, _
                0, ptr, 0)

            If result = 0 Then

                Dim ptrNext As IntPtr = ptr

                ' Enumerate all returned records
                Do Until ptrNext.Equals(IntPtr.Zero)

                    ' Get the record from the pointer
                    Dim record As UnsafeNativeMethods.DnsRecord = DirectCast(PtrToStructure(ptrNext, GetType(UnsafeNativeMethods.DnsRecord)), UnsafeNativeMethods.DnsRecord)

                    ' Check if the record type is A
                    If record.wType = UnsafeNativeMethods.DnsQueryTypes.DNS_TYPE_A Then

                        ' Add the ip address to the list
                        servers.Add(New System.Net.IPAddress(record.A.IpAddress))

                    End If

                    ' Get the pointer to the next record
                    ptrNext = record.pNext

                Loop

                ' Release the record list
                UnsafeNativeMethods.DnsRecordListFree(ptr, 0)

            Else

                ' Throw the exception
                Throw New Win32Exception(result)

            End If

            ' Return the ips
            Return DirectCast(servers.ToArray(GetType(System.Net.IPAddress)), System.Net.IPAddress())

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the mail exchanger servers of a domain.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetMailExchangers(ByVal domain As String) As MailExchanger()

            Dim ptr As IntPtr
            Dim result As Integer
            Dim servers As New ArrayList

            ' Query the DNS server for the MX records
            result = UnsafeNativeMethods.DnsQuery( _
                domain, _
                UnsafeNativeMethods.DnsQueryTypes.DNS_TYPE_MX, _
                UnsafeNativeMethods.DnsQueryOptions.DNS_QUERY_BYPASS_CACHE, _
                0, ptr, 0)

            If result = 0 Then

                Dim ptrNext As IntPtr = ptr

                ' Enumerate all returned records
                Do Until ptrNext.Equals(IntPtr.Zero)

                    ' Get the record from the pointer
                    Dim record As UnsafeNativeMethods.DnsRecord
                    record = DirectCast(PtrToStructure(ptrNext, GetType(UnsafeNativeMethods.DnsRecord)), UnsafeNativeMethods.DnsRecord)

                    ' Check if the record is a MX record
                    If record.wType = UnsafeNativeMethods.DnsQueryTypes.DNS_TYPE_MX Then

                        ' Add the server to the list
                        servers.Add(New MailExchanger(Marshal.PtrToStringAuto(record.MX.pNameExchange), record.MX.wPreference))

                    End If

                    ' Get the pointer to the next record
                    ptrNext = record.pNext

                Loop

                ' Release the record list
                UnsafeNativeMethods.DnsRecordListFree(ptr, 0)

            Else

                ' Throw the exception
                Throw New Win32Exception(result)

            End If

            ' Sort the servers in preference order
            servers.Sort()

            ' Return the server array
            Return DirectCast(servers.ToArray(GetType(MailExchanger)), MailExchanger())

        End Function

    End Class

End Namespace