Option Explicit On 
Option Strict On

Imports System
Imports System.Collections
Imports System.Runtime.Serialization

Namespace Edanmo.Net.Mail

    Public NotInheritable Class MailAttachmentCollection
        Inherits ReadOnlyCollectionBase

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Return wheter at least one attachment has a ContentDispotion = None.
        ''' </summary>
        ''' <remarks>
        ''' This method is used in the message to know if the message type has to
        ''' be set to multipart/related.
        ''' </remarks>
        ''' -----------------------------------------------------------------------------
        Friend Function IsRelated() As Boolean
            For i As Integer = 0 To innerlist.Count - 1
                If Item(i).ContentDisposition = ContentDisposition.None Then
                    Return True
                End If
            Next
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Adds an attachment to the collection.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Sub Add(ByVal attachment As MailAttachment)
            InnerList.Add(attachment)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Adds a file to the collection.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Sub Add(ByVal fileName As String)
            InnerList.Add(New MailAttachment(fileName))
        End Sub

        Public Sub CopyTo(ByVal array() As MailAttachment, ByVal index As Integer)
            InnerList.CopyTo(array, index)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Removes an attachment from the collection.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Sub Remove(ByVal index As Integer)
            InnerList.Remove(index)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets an attachment.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Default Public Property Item(ByVal index As Integer) As MailAttachment
            Get
                Return DirectCast(innerlist(index), MailAttachment)
            End Get
            Set(ByVal value As MailAttachment)
                innerlist(index) = value
            End Set
        End Property

        Public Sub Clear()
            innerlist.Clear()
        End Sub

    End Class

End Namespace
