Option Strict On
Option Explicit On 

Imports System
Imports System.Globalization
Imports System.Runtime.Serialization
Imports System.Security.Permissions

Namespace Edanmo.Net.Mail

    <Serializable()> _
    Public NotInheritable Class MailException
        Inherits System.Exception

        Private _errorCode As Integer

        Public Sub New(ByVal message As String)

            MyBase.New(message.Substring(4))

            _errorCode = Integer.Parse(message.Substring(0, 3), CultureInfo.InvariantCulture)

        End Sub

        Public Sub New(ByVal message As String, ByVal innerException As Exception)

            MyBase.New(message.Substring(4), innerException)

            _errorCode = Integer.Parse(message.Substring(0, 3), CultureInfo.InvariantCulture)

        End Sub

        Public Sub New()
        End Sub

        Private Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)

            MyBase.New(info, context)

            _errorCode = info.GetInt32("ErrorCode")

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets the error code returned by the smtp server
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property ErrorCode() As Integer
            Get
                Return _errorCode
            End Get
        End Property

        <SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter:=True)> _
        Public Overrides Sub GetObjectData(ByVal info As System.Runtime.Serialization.SerializationInfo, ByVal context As System.Runtime.Serialization.StreamingContext)

            MyBase.GetObjectData(info, context)

            info.AddValue("ErrorCode", _errorCode)

        End Sub

    End Class

End Namespace
