Option Explicit On 
Option Strict On

Imports System
Imports System.Collections
Imports System.Collections.Specialized
Imports System.Globalization
Imports System.Runtime.Serialization

Namespace Edanmo.Net.Mail

    Public NotInheritable Class MailHeaderCollection
        Inherits NameValueCollection

        Private Const HEADER_CONTENT_TRANSFER_ENCODING As String = "Content-Transfer-Encoding"
        Private Const HEADER_CONTENT_TYPE As String = "Content-Type"
        Private Const HEADER_DATE As String = "Date"
        Private Const HEADER_MIME_VERSION As String = "Mime-Version"
        Private Const HEADER_SUBJECT As String = "Subject"
        Private Const HEADER_X_MSMAIL_PRIORITY As String = "X-MSMail-Priority"

        Public Sub New()

            Add(HEADER_CONTENT_TYPE, "text/plain; charset=utf8")
            Add(HEADER_MIME_VERSION, "1.0")
            Add(HEADER_CONTENT_TRANSFER_ENCODING, "8bit")
            Add(HEADER_X_MSMAIL_PRIORITY, "Normal")
            Me.Date = Date.Now

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the Content-Transfer-Encoding header.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Property ContentTransferEncoding() As String
            Get
                Return Me.Get(HEADER_CONTENT_TRANSFER_ENCODING)
            End Get
            Set(ByVal value As String)
                Me.Set(HEADER_CONTENT_TRANSFER_ENCODING, value)
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the Content-Type header.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Property ContentType() As String
            Get
                Return Me.Get(HEADER_CONTENT_TYPE)
            End Get
            Set(ByVal value As String)
                Me.Set(HEADER_CONTENT_TYPE, value)
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the Date header.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Property [Date]() As Date
            Get
                Return Date.Parse(Me.Get(HEADER_DATE), CultureInfo.InvariantCulture, System.Globalization.DateTimeStyles.AllowWhiteSpaces).ToLocalTime
            End Get
            Set(ByVal value As Date)
                Me.Set(HEADER_DATE, value.ToUniversalTime.ToString("dd MMM yyyy hh:mm:ss' +0000'", CultureInfo.InvariantCulture))
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the Mime-Version header.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Property MimeVersion() As String
            Get
                Return Me.Get(HEADER_MIME_VERSION)
            End Get
            Set(ByVal value As String)
                Me.Set(HEADER_MIME_VERSION, value)
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the Priority headers.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Property Priority() As MailPriority
            Get
                Return DirectCast([Enum].Parse(GetType(MailPriority), Me.Get(HEADER_X_MSMAIL_PRIORITY)), MailPriority)
            End Get
            Set(ByVal value As MailPriority)
                Me.Set(HEADER_X_MSMAIL_PRIORITY, value.ToString)
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the Subject header.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Friend Property Subject() As String
            Get
                Return Me.Get(HEADER_SUBJECT)
            End Get
            Set(ByVal value As String)
                Me.Set(HEADER_SUBJECT, value)
            End Set
        End Property

        Public Overloads Sub CopyTo(ByVal array() As String, ByVal index As Integer)
            DirectCast(Me, ICollection).CopyTo(array, index)
        End Sub

    End Class

End Namespace