Option Explicit On 
Option Strict On

Imports System
Imports System.Globalization

Namespace Edanmo.Net

    ''' ----------------------------------------------------------------------------------------
    ''' <summary>
    ''' Represents a mail exchanger server
    ''' </summary>
    ''' ----------------------------------------------------------------------------------------
    Public Structure MailExchanger
        Implements IComparable

        Private _hostName As String
        Private _preference As Integer

        Friend Sub New(ByVal hostName As String, ByVal preference As Integer)
            _hostName = hostName
            _preference = preference
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets the server name.
        ''' </summary>
        ''' <history>
        ''' 	[Eduardo Morcillo]	06/05/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property HostName() As String
            Get
                Return _hostName
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets the server preference.
        ''' </summary>
        ''' <history>
        ''' 	[Eduardo Morcillo]	06/05/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property Preference() As Integer
            Get
                Return _preference
            End Get
        End Property

        Public Function CompareTo(ByVal obj As MailExchanger) As Integer

            Dim hostCompare As Integer

            hostCompare = _hostName.CompareTo(obj._hostName)

            If hostCompare = 0 Then
                Return _preference.CompareTo(obj._preference)
            Else
                Return hostCompare
            End If

        End Function

        Private Function CompareTo(ByVal obj As Object) As Integer Implements System.IComparable.CompareTo

            If TypeOf obj Is MailExchanger Then
                Return CompareTo(DirectCast(obj, MailExchanger))
            End If

            Throw New ArgumentException(ResourceHelper.GetString("MX_InvalidCompareToArgument"))

        End Function

        Public Overrides Function ToString() As String

            Return String.Format( _
                            CultureInfo.InvariantCulture, _
                            "{0}:{1}", _
                            _hostName, _preference)

        End Function

        Public Overloads Function Equals(ByVal obj As MailExchanger) As Boolean
            Return Me.CompareTo(obj) = 0
        End Function

        Public Overloads Overrides Function Equals(ByVal obj As Object) As Boolean

            If TypeOf obj Is MailExchanger Then
                Return Equals(DirectCast(obj, MailExchanger))
            End If

            Return False

        End Function

        Public Overrides Function GetHashCode() As Integer
            Return _hostName.GetHashCode Xor _preference.GetHashCode
        End Function

    End Structure

End Namespace
