Option Explicit On 
Option Strict On

Imports System
Imports System.Security.Cryptography

Namespace Edanmo.Security.Cryptography

    ''' -----------------------------------------------------------------------------
    ''' Project	 : Edanmo.Security
    ''' Class	 : Security.Cryptography.HMACMD5
    ''' 
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Computes a Hash-based Message Authentication Code (HMAC) using the System.Security.Cryptography.MD5 hash function.  
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public NotInheritable Class HMACMD5
        Inherits KeyedHashAlgorithm

        Private _hash1 As MD5
        Private _hash2 As MD5
        Private _hashing As Boolean
        Private _ipad(63) As Byte
        Private _opad(63) As Byte

        Public Sub New(ByVal key() As Byte)

            ' Create two hash objects
            _hash1 = MD5.Create()
            _hash2 = MD5.Create()

            Me.Key = key

        End Sub

        Public Sub New()

            ' Create two hash objects
            _hash1 = MD5.Create()
            _hash2 = MD5.Create()

            ' Create a random key
            Dim random As RandomNumberGenerator = RandomNumberGenerator.Create()
            Me.KeyValue = New Byte(63) {}
            random.GetBytes(Me.KeyValue)

            ' XOR ipad and opad with the key
            XorKey()

        End Sub

        Protected Overrides Function HashFinal() As Byte()

            ' If hash was not started yet
            ' add ipad to the hash
            If Not _hashing Then
                _hash1.TransformBlock(_ipad, 0, 64, _ipad, 0)
                _hashing = True
            End If

            _hash1.TransformFinalBlock(New Byte(-1) {}, 0, 0)

            ' Hash opad in the second hash
            Me._hash2.TransformBlock(_opad, 0, 64, _opad, 0)

            ' Hash the first hash
            Me._hash2.TransformFinalBlock(_hash1.Hash, 0, _hash1.Hash.Length)
            _hashing = False

            ' Return the hash
            Return Me._hash2.Hash

        End Function

        Protected Overrides Sub Dispose(ByVal disposing As Boolean)

            Try

                If disposing Then

                    If Not _hash1 Is Nothing Then DirectCast(_hash1, IDisposable).Dispose()
                    If Not _hash2 Is Nothing Then DirectCast(_hash2, IDisposable).Dispose()
                    If Not _ipad Is Nothing Then Array.Clear(_ipad, 0, _ipad.Length)
                    If Not _opad Is Nothing Then Array.Clear(_opad, 0, _opad.Length)

                End If

            Finally

                MyBase.Dispose(disposing)

            End Try

        End Sub

        Protected Overrides Sub HashCore(ByVal array() As Byte, ByVal ibStart As Integer, ByVal cbSize As Integer)

            ' If hash was not started yet
            ' add ipad to the hash
            If Not _hashing Then
                _hash1.TransformBlock(_ipad, 0, 64, _ipad, 0)
                _hashing = True
            End If

            ' Hash the array
            Me._hash1.TransformBlock(array, ibStart, cbSize, array, ibStart)

        End Sub

        Public Overrides Sub Initialize()
            _hash1.Initialize()
            _hash2.Initialize()
            _hashing = False
        End Sub

        Private Sub XorKey()

            ' Initialize ipad and opad
            For i As Integer = 0 To 63
                _ipad(i) = &H36
                _opad(i) = &H5C
            Next

            ' XOR ipad and opad with the key
            For i As Integer = 0 To 63
                _ipad(i) = _ipad(i) Xor KeyValue(i)
                _opad(i) = _opad(i) Xor KeyValue(i)
            Next

        End Sub

        Public Overrides Property Key() As Byte()
            Get
                Return DirectCast(Me.KeyValue.Clone, Byte())
            End Get
            Set(ByVal value() As Byte)

                If _hashing Then Throw New CryptographicException(ResourceHelper.GetString("CantChangeKey"))

                Me.KeyValue = New Byte(63) {}

                If (value.Length > 64) Then
                    Me._hash1.ComputeHash(value).CopyTo(KeyValue, 0)
                Else
                    value.CopyTo(KeyValue, 0)
                End If

                XorKey()

            End Set
        End Property

    End Class

End Namespace
